@@ -17,27 +17,29 @@
 
 void es_setVideoMode(sdl_data *sd, int len, char* bp) 
 {
-    char* start;
-    int w, h, bpp, type;
-    int sendlen;
-    SDL_Surface *screen;
-  
-    w    = get16be(bp);
-    h    = get16be(bp);
-    bpp  = get16be(bp);
-    type = get32be(bp);
+   if(!sd->use_smp) {
+       es_setVideoMode2(sd->driver_data, 
+			driver_caller(sd->driver_data), bp);
+   } else { /* opengl initialization must be called from thread */
+       gl_dispatch(sd, SDL_SetVideoModeFunc, len, bp);
+   }
+}
 
-    screen = SDL_SetVideoMode(w, h, bpp, type);
+void es_setVideoMode2(ErlDrvPort port, ErlDrvTermData caller, char* bp) 
+{
+   int w, h, bpp, type;
+   SDL_Surface *screen;
+   ErlDrvTermData rt[8];
 
-    if ((type & SDL_OPENGL) == SDL_OPENGL) {
-      init_glexts(sd);
-    }
-    
-    bp = start = sdl_get_temp_buff(sd, 8);
-    PUSHGLPTR(screen, bp);
-    
-    sendlen = bp - start;
-    sdl_send(sd, sendlen);
+   w    = get16be(bp);
+   h    = get16be(bp);
+   bpp  = get16be(bp);
+   type = get32be(bp);
+   screen = SDL_SetVideoMode(w, h, bpp, type),
+   rt[0] = ERL_DRV_ATOM; rt[1]=driver_mk_atom((char *) "_esdl_result_");  
+   rt[2] = ERL_DRV_UINT;  rt[3] = (ErlDrvUInt) screen;
+   rt[4] = ERL_DRV_TUPLE; rt[5] = 2;
+   driver_send_term(port,caller,rt,6);
 }
 
 void es_videoDriverName(sdl_data *sd, int len, char *buff)
@@ -97,13 +99,22 @@ void es_getVideoInfo(sdl_data *sd, int len, char *buff)
    sdl_send(sd, sendlen);
 }
 
-void es_videoModeOK(sdl_data *sd, int len, char *buff)
+void es_videoModeOK(sdl_data *sd, int len, char *bp)
 {   
-   char *bp, *start;
+   if(!sd->use_smp) {
+       es_videoModeOK2(sd->driver_data, driver_caller(sd->driver_data), bp);
+   } else { 
+       gl_dispatch(sd, SDL_VideoModeOKFunc, len, bp);
+   }
+}
+
+void es_videoModeOK2(ErlDrvPort port, ErlDrvTermData caller, char *buff)
+{   
+   char *bp;
    int w, h, bpp, type;
-   int sendlen;
    int res;
-  
+   ErlDrvTermData rt[8];
+
    bp = buff;
   
    w    = get16be(bp);
@@ -112,14 +123,13 @@ void es_videoModeOK(sdl_data *sd, int len, char *buff)
    type = get32be(bp);
   
    res = SDL_VideoModeOK(w,h,bpp,type);
-    
-   bp = start = sdl_get_temp_buff(sd, 1);
-   put8(bp, res);
-    
-   sendlen = bp - start;
-   sdl_send(sd, sendlen);
+   rt[0] = ERL_DRV_ATOM; rt[1]=driver_mk_atom((char *) "_esdl_result_");  
+   rt[2] = ERL_DRV_INT; rt[3] = res;
+   rt[4] = ERL_DRV_TUPLE; rt[5] = 2;
+   driver_send_term(port,caller,rt,6);
 }
 
+
 void es_listModes(sdl_data *sd, int len, char *buff)
 {
    char *bp, *start;
@@ -677,13 +687,22 @@ void es_getGammaRamp(sdl_data *sd, int len, char * buff)
 
 void es_wm_setCaption(sdl_data *sd, int len, char *bp) 
 { 
-  char* title;
-  char* icon;
+    if(!sd->use_smp) {
+	es_wm_setCaption2(bp);
+    } else { 
+	gl_dispatch(sd, SDL_WM_SetCaptionFunc, len, bp);
+    }
+}
 
-  title = bp;
-  icon = title + strlen(title) + 1;
-   
-  SDL_WM_SetCaption(title, icon);      
+void es_wm_setCaption2(char *bp) 
+{
+    char* title;
+    char* icon;
+
+    title = bp;
+    icon = title + strlen(title) + 1;
+    
+    SDL_WM_SetCaption(title, icon);
 }
 
 void es_wm_getCaption(sdl_data *sd, int len, char *buff) 
@@ -728,20 +747,29 @@ void es_wm_iconifyWindow(sdl_data *sd, int len, char *buff)
    SDL_WM_IconifyWindow();     
 }
 
-void es_wm_toggleFullScreen(sdl_data *sd, int len, char *buff) 
+void es_wm_toggleFullScreen(sdl_data *sd, int len, char *bp) 
 {
-   char *bp, *start;
-   SDL_Surface *sptr;
-   int res, sendlen;
-  
-   bp = buff;   
-   POPGLPTR(sptr, bp);
-   res = SDL_WM_ToggleFullScreen(sptr);
-   bp = start = sdl_get_temp_buff(sd, 1);
-   
-   put8(bp, res); 
-   sendlen = bp - start;
-   sdl_send(sd, sendlen);
+    if(!sd->use_smp) {
+	es_wm_toggleFullScreen2(sd->driver_data, driver_caller(sd->driver_data), bp);
+    } else {
+	gl_dispatch(sd, SDL_WM_ToggleFullScreenFunc, len, bp);
+    }
+}
+
+void es_wm_toggleFullScreen2(ErlDrvPort port, ErlDrvTermData caller, char *buff) 
+{
+    char *bp;
+    SDL_Surface *sptr;
+    int res;
+    ErlDrvTermData rt[8];
+
+    bp = buff;   
+    POPGLPTR(sptr, bp);
+    res = SDL_WM_ToggleFullScreen(sptr);
+    rt[0] = ERL_DRV_ATOM; rt[1]=driver_mk_atom((char *) "_esdl_result_");  
+    rt[2] = ERL_DRV_INT; rt[3] = res;
+    rt[4] = ERL_DRV_TUPLE; rt[5] = 2;
+    driver_send_term(port,caller,rt,6);
 }
 
 
@@ -759,35 +787,39 @@ void es_wm_grabInput(sdl_data *sd, int len, char *bp)
   sdl_send(sd, sendlen);
 }
 
-void es_wm_getInfo(sdl_data *sd, int len, char *buff) 
+void es_wm_getInfo(sdl_data *sd, int len, char *bp) 
 {
-   char *bp, *start;
+    if(!sd->use_smp) {
+	es_wm_getInfo2(sd->driver_data, driver_caller(sd->driver_data), bp);
+    } else {
+	gl_dispatch(sd, SDL_WM_GetInfoFunc, len, bp);
+    }
+}
+
+void es_wm_getInfo2(ErlDrvPort port, ErlDrvTermData caller, char *buff) 
+{
+   int res;
    SDL_SysWMinfo info;
-   int sendlen;
+   ErlDrvTermData rt[12];
 
-   bp = buff;   
    SDL_VERSION(&info.version);
    SDL_GetWMInfo(&info);
-   bp = start = sdl_get_temp_buff(sd, 7);
-/*    fprintf(stderr, "es_wm_getInfo %d %d %d %d %d \r\n",  */
-/* 	   (unsigned int *) &info, (unsigned int*) &info.version, */
-/* 	   info.version.major, */
-/* 	   info.version.minor, */
-/* 	   info.version.patch); */
 
-   put8(bp, info.version.major);
-   put8(bp, info.version.minor);
-   put8(bp, info.version.patch);
+   rt[0] = ERL_DRV_ATOM; rt[1]=driver_mk_atom((char *) "_esdl_result_");  
+   rt[2] = ERL_DRV_INT; rt[3] = info.version.major;
+   rt[4] = ERL_DRV_INT; rt[5] = info.version.minor;
+   rt[6] = ERL_DRV_INT; rt[7] = info.version.patch;
 #ifdef WIN32
-   if(sizeof(info.window) == 4)
-      put32be(bp, (unsigned int) info.window);
-   else
-      put32be(bp, 0);
+   if(sizeof(info.window) == 4) {
+       rt[8] = ERL_DRV_UINT; rt[9] = (unsigned int) info.window;
+   } else {
+       rt[8] = ERL_DRV_UINT; rt[9] = 0;
+   }
 #else
-   put32be(bp, 0);
+   rt[8] = ERL_DRV_UINT; rt[9] = 0;
 #endif
-   sendlen = bp - start;
-   sdl_send(sd, sendlen);
+   rt[10] = ERL_DRV_TUPLE; rt[11] = 5;
+   res = driver_send_term(port,caller,rt,12);
 }
 
 void es_wm_isMaximized(sdl_data *sd, int len, char *buff)
@@ -801,7 +833,7 @@ void es_wm_isMaximized(sdl_data *sd, int len, char *buff)
 	SDL_SysWMinfo info;
 	
 	SDL_VERSION(&info.version);
-	SDL_GetWMInfo(&info);     
+	SDL_GetWMInfo(&info);
 	s = IsZoomed(info.window);
      }
 #else
@@ -832,7 +864,38 @@ void es_wm_isMaximized(sdl_data *sd, int len, char *buff)
      sdl_send(sd, sendlen);
 }
 
-void es_gl_setAttribute(sdl_data *sd, int len, char *buff)
+void es_wm_maximize(sdl_data *sd, int len, char *bp)
+{
+    if(!sd->use_smp) {
+	es_wm_maximize2(sd->driver_data, 
+			driver_caller(sd->driver_data), bp);
+    } else { 
+	gl_dispatch(sd, SDL_WM_MaximizeFunc, len, bp);
+    }
+}
+
+void es_wm_maximize2(ErlDrvPort port, ErlDrvTermData caller, char *buff)
+{
+#ifdef _WIN32
+    SDL_SysWMinfo info;
+    SDL_GetWMInfo(&info);
+    ShowWindow(info.window, SW_SHOWMAXIMIZED);
+#endif
+}
+
+
+
+void es_gl_setAttribute(sdl_data *sd, int len, char *bp)
+{
+    if(!sd->use_smp) {
+	es_gl_setAttribute2(sd->driver_data, 
+			    driver_caller(sd->driver_data), bp);
+    } else { 
+	gl_dispatch(sd, SDL_GL_SetAttributeFunc, len, bp);
+    }
+}
+
+void es_gl_setAttribute2(ErlDrvPort port, ErlDrvTermData caller, char *buff)
 {
     char *bp;
     int attr, val;
@@ -842,28 +905,50 @@ void es_gl_setAttribute(sdl_data *sd, int len, char *buff)
     SDL_GL_SetAttribute(attr, val);
 }
 
-void es_gl_getAttribute(sdl_data *sd, int len, char *buff)
+
+void es_gl_getAttribute(sdl_data *sd, int len, char *bp)
 {
-    char *bp, *start;
-    int attr, val, sendlen;
+    if(!sd->use_smp) {
+	es_gl_getAttribute2(sd->driver_data, 
+			    driver_caller(sd->driver_data), bp);
+    } else { 
+	gl_dispatch(sd, SDL_GL_GetAttributeFunc, len, bp);
+    }
+}
+
+void es_gl_getAttribute2(ErlDrvPort port, ErlDrvTermData caller, char *buff)
+{
+    char *bp;
+    int attr, val;
+    ErlDrvTermData rt[8];
+
     bp = buff;
     attr = get16be(bp);
     SDL_GL_GetAttribute(attr, &val);
-    bp = start = sdl_get_temp_buff(sd, 4);
-    put32be(bp, val);
-    sendlen = bp - start;
-    sdl_send(sd, sendlen);
+
+    rt[0] = ERL_DRV_ATOM; rt[1]=driver_mk_atom((char *) "_esdl_result_");  
+    rt[2] = ERL_DRV_INT; rt[3] = val;
+    rt[4] = ERL_DRV_TUPLE; rt[5] = 2;
+    driver_send_term(port,caller,rt,6);
 }
 
+
 void es_gl_swapBuffers(sdl_data *sd, int len, char *buff)
-{   
+{  
    char *bp, *start;
    int sendlen;
    Uint32 ts;
-
-   SDL_GL_SwapBuffers();
+   
+   if(!sd->use_smp) 
+      SDL_GL_SwapBuffers();
+   else
+      gl_dispatch(sd, SDL_GL_SwapBuffersFunc, len, buff);
+   
+   /* Timer is removed it a bad idea anyway 
+      (works if vsync is on only I believe) */
+   
    bp = start = sdl_get_temp_buff(sd, 4);
-   ts = SDL_GetTicks();
+   ts = 0; 
    put32be(bp, ts);
    sendlen = bp - start;
    sdl_send(sd, sendlen);
