@@ -14,9 +14,10 @@
 
 -module(sdl_video).
 
+-include("sdl.hrl").
+-include("esdl.hrl").
 -include("sdl_video.hrl").
 -include("sdl_video_funcs.hrl").
--include("gl.hrl").
 -include("sdl_util.hrl").
 
 %%% ERL_SDL functions
@@ -67,6 +68,7 @@
 	 wm_grabInput/1,
 	 wm_getInfo/0,
 	 wm_isMaximized/0,
+	 wm_maximize/0,
 	 %% OpenGL Support funcs 
 	 gl_setAttribute/2,
 	 gl_getAttribute/1,
@@ -188,9 +190,28 @@ getPixels({surfacep, Ref}, Size) ->
 %% C-API func:  SDL_Surface *SDL_SetVideoMode(int width, int height, 
 %%                                            int bpp, Uint32 flags);
 setVideoMode(W, H, Bpp, Type) ->
-    case call(?SDL_SetVideoMode, <<W:16,H:16,Bpp:16,Type:32>>) of
-	<<0:?_PTR>> -> error;
-	<<Res:?_PTR>> -> {surfacep, Res}
+    call(?SDL_SetVideoMode, <<W:16,H:16,Bpp:16,Type:32>>),
+    receive 
+	{'_esdl_result_', 0} -> error;
+	{'_esdl_result_', Pointer} -> 
+	    case (Type band ?SDL_OPENGL) > 1 of
+		true ->
+		    DynLib = case os:type() of
+				 {win32,_} -> "erl_gl.dll";
+				 _ ->         "erl_gl.so"
+			     end,		    
+		    WXDL = list_to_binary(filename:join(code:priv_dir(wx),DynLib)),
+		    call(?ESDL_Init_Opengl,  <<(WXDL)/binary, 0:8>>),
+		    receive 
+			{'_esdl_result_', 0} ->
+			    error(opengl_init_failed);
+			{'_esdl_result_', GL} when GL > 0 ->
+			    put(opengl_port, esdl_port),
+			    ok
+		    end;
+		false -> ok
+	    end,
+	    {surfacep, Pointer}
     end.
 
 %% Func: videoDriverName
@@ -230,8 +251,10 @@ getVideoinfo() ->
 %% Returns:  true | false  (see sdl doc)
 %% C-API func: int SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags);
 videoModeOK(W, H, Bpp, Type) ->
-    <<Res:8>> = call(?SDL_VideoModeOK, <<W:16, H:16, Bpp:16, Type:32>>),
-    Res /= 0.				  
+    call(?SDL_VideoModeOK, <<W:16, H:16, Bpp:16, Type:32>>),
+    receive
+	{'_esdl_result_', Res} -> Res /= 0
+    end.
 
 %% Func: listModes
 %% Args: PixelFormat (either as sdl_pixelformat record or a PixelFormat Ref or null) 
@@ -568,7 +591,7 @@ setGamma(Red,Green,Blue) ->
 %% C-API func: int SDL_SetGammaRamp(Uint16 red[256], Uint16 green[256], Uint16 blue[256])
 %% Desc: 
 setGammaRamp(Red,Green,Blue) ->
-    Args = sdl_util:term2bin(Red ++ Green ++ Blue, 256*3, ?GL_UNSIGNED_SHORT),
+    Args = sdl_util:term2bin(Red ++ Green ++ Blue, 256*3, ?SDL_UNSIGNED_SHORT),
     <<Res:32>> = call(?SDL_SetGammaRamp, Args),
     Res == 0.
 
@@ -580,9 +603,9 @@ setGammaRamp(Red,Green,Blue) ->
 getGammaRamp() ->
     <<Res:32, R:512/binary,G:512/binary,B:512/binary>> = call(?SDL_GetGammaRamp, []),
     {Res == 0, 
-     sdl_util:bin2list(256, ?GL_UNSIGNED_SHORT, R),  
-     sdl_util:bin2list(256, ?GL_UNSIGNED_SHORT, G),  
-     sdl_util:bin2list(256, ?GL_UNSIGNED_SHORT, B)}.
+     sdl_util:bin2list(256, ?SDL_UNSIGNED_SHORT, R),  
+     sdl_util:bin2list(256, ?SDL_UNSIGNED_SHORT, G),  
+     sdl_util:bin2list(256, ?SDL_UNSIGNED_SHORT, B)}.
 
 %%%%%%%%%%% WindowManager Functions %%%%%%%%%%%
 
@@ -627,8 +650,10 @@ wm_iconifyWindow() ->
 wm_toggleFullScreen(Surface) when is_record(Surface, sdl_surface) ->
     wm_toggleFullScreen(Surface#sdl_surface.self);
 wm_toggleFullScreen({surfacep, Ref}) ->
-    <<Res:8>> = call(?SDL_WM_ToggleFullScreen, <<Ref:?_PTR>>),
-    Res == 1.
+    cast(?SDL_WM_ToggleFullScreen, <<Ref:?_PTR>>),
+    receive
+	{'_esdl_result_', Res} ->  Res == 1
+    end.
 
 %% Func:   wm_grabInput
 %% Args:   GrabMode (?SDL_GRAB_QUERY | ?SDL_GRAB_OFF | ?SDL_GRAB_ON)
@@ -644,9 +669,11 @@ wm_grabInput(GrabMode) ->
 %% C-API func: 
 %% Desc:  Only windows returns anything useful (window handle)
 wm_getInfo() ->
-    <<Major:8, Minor:8, Patch:8, Rest/binary>> 
-	= call(?SDL_WM_GetInfo, []),
-    {{Major, Minor, Patch}, Rest}.
+    cast(?SDL_WM_GetInfo, []),
+    receive 
+	{'_esdl_result_', Major, Minor, Patch, Rest} ->
+	    {{Major, Minor, Patch}, <<Rest:32/unsigned-native>>}
+    end.
 
 %% Func:   wm_isMaximized
 %% Args:   
@@ -657,6 +684,14 @@ wm_isMaximized() ->
     <<Bool:8>> = call(?SDL_WM_IsMaximized, []),
     Bool /= 0.
 
+%% Func:   wm_maximize
+%% Args:   
+%% Returns: nothing
+%% C-API func: 
+%% Desc:  Only works on windows 
+wm_maximize() ->
+    cast(?SDL_WM_Maximize, []).
+
 %%%%%%%%%%%%%%%%%% GL support Funcs %%%%%%%%%%%%%%%%%%%%%
 
 %% Func:    gl_setAttribute
@@ -671,8 +706,11 @@ gl_setAttribute(Attr, Val) ->
 %% Returns: Attribute Value
 %% C-API func: int SDL_GL_GetAttribute(SDL_GLattr attr, int* value);
 gl_getAttribute(Attr) ->
-    <<Res:32>> = call(?SDL_GL_GetAttribute, <<Attr:16>>),
-    Res.
+    call(?SDL_GL_GetAttribute, <<Attr:16>>),
+    receive 
+	{'_esdl_result_', Res} ->
+	    Res
+    end.
 
 %% Func:    gl_swapBuffers
 %% Args:    
